{--------------------------------------------------------------------------
Sprache    : Delphi 1.0
Aufgaben   : Funktionen und Prozeduren des inkrementellen Tokenizers
Hinweise   : Programme, die diese Unit benutzen, muessen mit der
             Anweisung $X+ compiliert werden
Fehler     : Fehler bitte melden an: scheffczyk@verwaltung.uni-mainz.de
             InitToken     = Initialisiert einen inkrementellen Tokenizer und
                             ermoeglicht damit den Aufruf der Funktion
                             'NextToken'
             ClearToken    = Loescht alle Einstellungen des inkrementellen
                             Tokenizers und verhindert den weiteren Aufruf
                             der Funktion 'NextToken'
             NextToken     = Gibt das naechste Token und das naechste
                             Trennzeichen zurueck (Zuvor muss eine
                             Initialisierung durch die Funktion 'InitToken'
                             stattgefunden haben.
             StrInitToken  = Initialisiert einen inkrementellen Tokenizer und
                             ermoeglicht damit den Aufruf der Funktion
                             'StrNextToken'
             StrClearToken = Loescht alle Einstellungen des inkrementellen
                             Tokenizers und verhindert den weiteren Aufruf
                             der Funktion 'StrNextToken'
             StrNextToken  = Gibt das naechste Token und das naechste
                             Trennzeichen zurueck (Zuvor muss eine
                             Initialisierung durch die Funktion 'StrInitToken'
                             stattgefunden haben.

---------------------------------------------------------------------------}
unit _4inctok;

{$X+}

{==========================================================================}
interface

{--------------------------------------------------------------------------
Name      : InitToken
Sprache   : Delphi 1.0
Version   : 2.0
Datum     : 27.03.1996
Autor     : Scheffczyk
Aufgabe   : Initialisiert den inkrementellen Tokenizer
Parameter : SourceStr : String  : der zu durchsuchende String
            StringSep : String  : der Seperatorzeichen-String
Rueckgabe :             Boolean : true : wenn die Initialisierung durchgefuehrt
                                         werden koennte
                                  false: wenn die Initialisierung nicht
                                         durchgefuehrt werden konnte, d.h. wenn
                                         der Tokenizer schon initialisiert ist
Fehler    :
Hinweise  : Es kann in einem Programm immer nur eine Initialisierung zur
            selben Zeit aktiv sein.
---------------------------------------------------------------------------}
function InitToken(aStr,aSep:String):Boolean;

{--------------------------------------------------------------------------
Name      : ClearToken
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 27.03.1996
Autor     : Scheffczyk
Aufgabe   : Setzt den inkrementellen Tokenizer zurueck
Parameter :
Rueckgabe :             Boolean : true : wenn die Ruecksetzung durchgefuehrt
                                         werden koennte
Fehler    :
Hinweise  : Da in einem Programm immer nur eine Initialisierung zur
            selben Zeit aktiv sein setzt ein Aufruf dieser Funktion unabhaengig
            von der Aufrufposition den Tokenizer zurueck
---------------------------------------------------------------------------}
function ClearToken:Boolean;

{--------------------------------------------------------------------------
Name      : NextToken
Sprache   : Delphi 1.0
Version   : 2.0
Datum     : 27.03.1996
Autor     : Scheffczyk
Aufgabe   : Liefert den naechsten Substring und das naechste Separatorzeichen
Parameter : theToken   : String  : der naechste Substring
            theSepChar : Char    : das naechste Separatorzeichen
Rueckgabe :              Boolean : true  : wenn ein weiterer Substring gefunden
                                           wurde
                                   false : wenn kein weiterer Substring
                                           gefunden werden konnte
Fehler    :
Hinweise  : Wenn kein Substring oder Separatorzeichen gefunden werden konnte
            werden ein Leerstring und das ASCII-Zeichen 0 zurueckgegeben
---------------------------------------------------------------------------}
function NextToken(var aToken, aSepChar: String):Boolean;

{--------------------------------------------------------------------------
Name      : StrInitToken
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 27.08.1995
Autor     : Scheffczyk
Aufgabe   : Initialisiert den inkrementellen Tokenizer
Parameter : SourceStr : PChar   : Zeiger auf den zu durchsuchenden String
            StringSep : PChar   : Zeiger auf den Seperatorzeichen-String
Rueckgabe :             Boolean : true : wenn die Initialisierung durchgefuehrt
                                         werden koennte
                                  false: wenn die Initialisierung nicht
                                         durchgefuehrt werden konnte, d.h. wenn
                                         der Tokenizer schon initialisiert ist
Fehler    :
Hinweise  : Es kann in einem Programm immer nur eine Initialisierung zur
            selben Zeit aktiv sein.
---------------------------------------------------------------------------}
function StrInitToken(aPChar:PChar; aSepChars : PChar):Boolean;

{--------------------------------------------------------------------------
Name      : StrClearToken
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 27.08.1995
Autor     : Scheffczyk
Aufgabe   : Setzt den inkrementellen Tokenizer zurueck
Parameter :
Rueckgabe :             Boolean : true : wenn die Ruecksetzung durchgefuehrt
                                         werden koennte
Fehler    :
Hinweise  : Da in einem Programm immer nur eine Initialisierung zur
            selben Zeit aktiv sein setzt ein Aufruf dieser Funktion unabhaengig
            von der Aufrufposition den Tokenizer zurueck
---------------------------------------------------------------------------}
function StrClearToken:Boolean;

{--------------------------------------------------------------------------
Name      : StrNextToken
Sprache   : Delphi 1.0
Version   : 1.0
Datum     : 27.08.1995
Autor     : Scheffczyk
Aufgabe   : Liefert den naechsten Substring und das naechste Separatorzeichen
Parameter : theToken   : PChar   : Zeiger auf den naechsten Substring
            theSepChar : Char    : das naechste Separatorzeichen
Rueckgabe :              Boolean : true  : wenn ein weiterer Substring gefunden
                                           wurde
                                   false : wenn kein weiterer Substring
                                           gefunden werden konnte
Fehler    :
Hinweise  : Wenn kein Substring oder Separatorzeichen gefunden werden konnte
            werden ein Leerstring und das ASCII-Zeichen 0 zurueckgegeben
---------------------------------------------------------------------------}
function StrNextToken(var theToken:PChar; var theSepChar: Char):Boolean;


{==========================================================================}
implementation

uses
    sysutils,
    _4token;

var
   PCharAnchor    : PChar;
   PCharWork      : PChar;
   SepCharsAnchor : PChar;
   SepCharsWork   : PChar;
   aString : String;
   aSepStr : String;
   TCount  : Byte;
   aPos    : Byte;
   aLenStr : Byte;
   aLenSep : Byte;

{---------------------------------------------------------------------------}
function StrInitToken(aPChar:PChar; aSepChars : PChar):Boolean;
begin
     if PCharAnchor = NIL then
     begin
          PCharAnchor    := StrAlloc(StrLen(aPChar)+1);
          PCharWork      := PCharAnchor;
          SepCharsAnchor := StrAlloc(StrLen(aSepChars)+1);
          SepCharsWork   := SepCharsAnchor;
          StrCopy(PCharAnchor,aPChar);
          StrCopy(SepCharsAnchor,aSepChars);
          Result         := true;
     end
     else
     begin
          Result := false;
     end;
end;

{---------------------------------------------------------------------------}
function StrClearToken:Boolean;

begin
     StrDispose(PCharAnchor);
     StrDispose(SepCharsAnchor);
     PCharAnchor      := NIL;
     PCharWork        := NIL;
     SepCharsAnchor   := NIL;
     SepCharsWork     := NIL;
     Result           := true;
end;

{---------------------------------------------------------------------------}
function StrNextToken(var theToken:PChar; var theSepChar : Char):Boolean;
begin
     if ((StrLen(PCharWork) = 0) or (theSepChar = #0)) then
     begin
          Result     := false;
     end
     else
     begin
          theToken   := StrGetToken(PCharWork,SepCharsWork,theToken,1);
          theSepChar := StrGetTSep(PCharWork,SepCharsWork,1);
          PCharWork  := PCharWork + StrLen(theToken) +1;
          Result     := true;
     end;
end;

{---------------------------------------------------------------------------}
function InitToken(aStr,aSep:String):Boolean;
begin
     aString := aStr;
     aSepStr := aSep;
     aPos    := 1;
     TCount  := 1;
     aLenStr := Length(aString);
     aLenSep := Length(aSepStr);
     Result  := true;
end;

{---------------------------------------------------------------------------}
function ClearToken:Boolean;
begin
     aString := '';
     aSepStr := '';
     TCount  := 1;
     aPos    := 0;
     aLenStr := 0;
     aLenSep := 0;
     Result  := true;
end;

{---------------------------------------------------------------------------}
function NextToken(var aToken, aSepChar: String):Boolean;
var
   i,j     : Byte;
   BoT,EoT : Byte;
   isSep   : Boolean;

begin
     BoT    := aPos;
     EoT    := aPos;
     for i := aPos to aLenStr do
     begin
          IsSep := false;
          for j := 1 to aLenSep do
          begin
               if aString[i] = aSepStr[j] then
               begin
                    IsSep := true;
                    Break;
               end;
          end;
          if IsSep then
          begin
               EoT  := i;
               aPos := i+1;
               aSepChar := aString[i];
               Break;
          end
          else
          begin
               if i = aLenStr then
               begin
                    EoT  := i;
                    aPos := i;
                    Break;
               end;
          end;
     end;
     if aPos < aLenStr then
     begin
          aToken := Copy(aString,BoT,EoT-BoT);
          Result := true;
     end
     else
     begin
          if aPos = aLenStr then
          begin
               aToken := Copy(aString,BoT,EoT-BoT+1);
               Result := true;
               aPos   := aPos +1;
          end
          else
          begin
               Result := false;
          end;
     end;
end;

{==========================================================================}
initialization

   ClearToken;
   PCharAnchor    := NIL;
   PCharWork      := NIL;
   SepCharsAnchor := NIL;
   SepCharsWork   := NIL;
end.
